--[[
    Character data handling
]]

local _, Internal = ...;

local UnitSex = UnitSex
local UnitRace = UnitRace
local UnitClass = UnitClass
local UnitFullName = UnitFullName
local GetRealmName = GetRealmName
local GetClassInfo = GetClassInfo
local GetNumClasses = GetNumClasses
local GetTalentInfoByID = GetTalentInfoByID
local GetSpecialization = GetSpecialization
local GetPvpTalentInfoByID = GetPvpTalentInfoByID
local GetPvpTalentSlotInfo = C_SpecializationInfo.GetPvpTalentSlotInfo;
local GetNumSpecializations = GetNumSpecializations
local GetSpecializationInfo = GetSpecializationInfo
local GetTalentInfoBySpecialization = GetTalentInfoBySpecialization
local GetNumSpecializationsForClassID = GetNumSpecializationsForClassID
local GetSpecializationInfoForClassID = GetSpecializationInfoForClassID
local GetEssenceInfo = C_AzeriteEssence.GetEssenceInfo;

local roles = {"TANK", "HEALER", "DAMAGER"};
local roleIndexes = {["TANK"] = 1, ["HEALER"] = 2, ["DAMAGER"] = 3};
local classInfo = {};
function Internal.Roles()
    return ipairs(roles)
end
function Internal.IsClassRoleValid(classFile, role)
	return classInfo[classFile][role] and true or false;
end
function Internal.UpdateClassInfo()
    for classIndex=1,GetNumClasses() do
        local className, classFile, classID = GetClassInfo(classIndex);
        classInfo[classFile] = {};
        for specIndex=1,GetNumSpecializationsForClassID(classID) do
            local role = select(5, GetSpecializationInfoForClassID(classID, specIndex));
            classInfo[classFile][role] = true;
        end
    end
end

local GetTalentInfoForSpecID;
local GetPvPTrinketTalentInfo;
local GetPvPTalentInfoForSpecID;
do
	local specInfo = {
		[62] = {
			["talents"] = {
				{
					22458, -- [1]
					22461, -- [2]
					22464, -- [3]
				}, -- [1]
				{
					23072, -- [1]
					22443, -- [2]
					16025, -- [3]
				}, -- [2]
				{
					22444, -- [1]
					22445, -- [2]
					22447, -- [3]
				}, -- [3]
				{
					22453, -- [1]
					22467, -- [2]
					22470, -- [3]
				}, -- [4]
				{
					22907, -- [1]
					22448, -- [2]
					22471, -- [3]
				}, -- [5]
				{
					22455, -- [1]
					22449, -- [2]
					22474, -- [3]
				}, -- [6]
				{
					21630, -- [1]
					21144, -- [2]
					21145, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3580, -- [1]
				3579, -- [2]
				3578, -- [3]
			},
			["pvptalents"] = {
				3442, -- [1]
				61, -- [2]
				635, -- [3]
				636, -- [4]
				3517, -- [5]
				3531, -- [6]
				3523, -- [7]
				3529, -- [8]
				62, -- [9]
				637, -- [10]
			},
		},
		[63] = {
			["talents"] = {
				{
					22456, -- [1]
					22459, -- [2]
					22462, -- [3]
				}, -- [1]
				{
					23071, -- [1]
					22443, -- [2]
					23074, -- [3]
				}, -- [2]
				{
					22444, -- [1]
					22445, -- [2]
					22447, -- [3]
				}, -- [3]
				{
					22450, -- [1]
					22465, -- [2]
					22468, -- [3]
				}, -- [4]
				{
					22904, -- [1]
					22448, -- [2]
					22471, -- [3]
				}, -- [5]
				{
					22451, -- [1]
					23362, -- [2]
					22472, -- [3]
				}, -- [6]
				{
					21631, -- [1]
					22220, -- [2]
					21633, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3583, -- [1]
				3582, -- [2]
				3581, -- [3]
			},
			["pvptalents"] = {
				648, -- [1]
				53, -- [2]
				56, -- [3]
				647, -- [4]
				3530, -- [5]
				646, -- [6]
				3524, -- [7]
				645, -- [8]
				644, -- [9]
				643, -- [10]
				828, -- [11]
			},
		},
		[250] = {
			["talents"] = {
				{
					19165, -- [1]
					19166, -- [2]
					19217, -- [3]
				}, -- [1]
				{
					19218, -- [1]
					19219, -- [2]
					19220, -- [3]
				}, -- [2]
				{
					19221, -- [1]
					22134, -- [2]
					22135, -- [3]
				}, -- [3]
				{
					22013, -- [1]
					22014, -- [2]
					22015, -- [3]
				}, -- [4]
				{
					19227, -- [1]
					19226, -- [2]
					19228, -- [3]
				}, -- [5]
				{
					19230, -- [1]
					19231, -- [2]
					19232, -- [3]
				}, -- [6]
				{
					21207, -- [1]
					21208, -- [2]
					21209, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3467, -- [1]
				3468, -- [2]
				3466, -- [3]
			},
			["pvptalents"] = {
				609, -- [1]
				608, -- [2]
				607, -- [3]
				841, -- [4]
				206, -- [5]
				205, -- [6]
				204, -- [7]
				3436, -- [8]
				3441, -- [9]
				3434, -- [10]
				3511, -- [11]
			},
		},
		[251] = {
			["talents"] = {
				{
					22016, -- [1]
					22017, -- [2]
					22018, -- [3]
				}, -- [1]
				{
					22019, -- [1]
					22020, -- [2]
					22021, -- [3]
				}, -- [2]
				{
					22515, -- [1]
					22517, -- [2]
					22519, -- [3]
				}, -- [3]
				{
					22521, -- [1]
					22523, -- [2]
					22525, -- [3]
				}, -- [4]
				{
					22527, -- [1]
					22530, -- [2]
					23373, -- [3]
				}, -- [5]
				{
					22531, -- [1]
					22533, -- [2]
					22535, -- [3]
				}, -- [6]
				{
					22023, -- [1]
					22109, -- [2]
					22537, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3538, -- [1]
				3539, -- [2]
				3540, -- [3]
			},
			["pvptalents"] = {
				3515, -- [1]
				3512, -- [2]
				3435, -- [3]
				43, -- [4]
				3743, -- [5]
				3439, -- [6]
				3742, -- [7]
				3749, -- [8]
				701, -- [9]
				702, -- [10]
				706, -- [11]
			},
		},
		[252] = {
			["talents"] = {
				{
					22024, -- [1]
					22025, -- [2]
					22026, -- [3]
				}, -- [1]
				{
					22027, -- [1]
					22028, -- [2]
					22029, -- [3]
				}, -- [2]
				{
					22516, -- [1]
					22518, -- [2]
					22520, -- [3]
				}, -- [3]
				{
					22522, -- [1]
					22524, -- [2]
					22526, -- [3]
				}, -- [4]
				{
					22528, -- [1]
					22529, -- [2]
					23373, -- [3]
				}, -- [5]
				{
					22532, -- [1]
					22534, -- [2]
					22536, -- [3]
				}, -- [6]
				{
					22030, -- [1]
					22110, -- [2]
					22538, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3537, -- [1]
				3536, -- [2]
				3535, -- [3]
			},
			["pvptalents"] = {
				152, -- [1]
				3748, -- [2]
				40, -- [3]
				41, -- [4]
				3746, -- [5]
				3437, -- [6]
				42, -- [7]
				149, -- [8]
				163, -- [9]
				3747, -- [10]
				3754, -- [11]
				3440, -- [12]
			},
		},
		[253] = {
			["talents"] = {
				{
					22291, -- [1]
					22280, -- [2]
					22282, -- [3]
				}, -- [1]
				{
					22500, -- [1]
					22266, -- [2]
					22290, -- [3]
				}, -- [2]
				{
					19347, -- [1]
					19348, -- [2]
					23100, -- [3]
				}, -- [3]
				{
					22441, -- [1]
					22347, -- [2]
					22269, -- [3]
				}, -- [4]
				{
					22268, -- [1]
					22276, -- [2]
					22499, -- [3]
				}, -- [5]
				{
					19357, -- [1]
					22002, -- [2]
					23044, -- [3]
				}, -- [6]
				{
					22273, -- [1]
					21986, -- [2]
					22295, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3562, -- [1]
				3561, -- [2]
				3560, -- [3]
			},
			["pvptalents"] = {
				821, -- [1]
				824, -- [2]
				825, -- [3]
				1214, -- [4]
				693, -- [5]
				3730, -- [6]
				3612, -- [7]
				3605, -- [8]
				3604, -- [9]
				3603, -- [10]
				3602, -- [11]
				3600, -- [12]
				3599, -- [13]
			},
		},
		[254] = {
			["talents"] = {
				{
					22279, -- [1]
					22501, -- [2]
					22289, -- [3]
				}, -- [1]
				{
					22495, -- [1]
					22497, -- [2]
					22498, -- [3]
				}, -- [2]
				{
					19347, -- [1]
					19348, -- [2]
					23100, -- [3]
				}, -- [3]
				{
					22267, -- [1]
					22286, -- [2]
					21998, -- [3]
				}, -- [4]
				{
					22268, -- [1]
					22276, -- [2]
					22499, -- [3]
				}, -- [5]
				{
					23063, -- [1]
					23104, -- [2]
					22287, -- [3]
				}, -- [6]
				{
					22274, -- [1]
					22308, -- [2]
					22288, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3564, -- [1]
				3563, -- [2]
				3565, -- [3]
			},
			["pvptalents"] = {
				651, -- [1]
				652, -- [2]
				653, -- [3]
				659, -- [4]
				654, -- [5]
				3614, -- [6]
				658, -- [7]
				649, -- [8]
				660, -- [9]
				656, -- [10]
				3729, -- [11]
				657, -- [12]
			},
		},
		[255] = {
			["talents"] = {
				{
					22275, -- [1]
					22283, -- [2]
					22296, -- [3]
				}, -- [1]
				{
					21997, -- [1]
					22769, -- [2]
					22297, -- [3]
				}, -- [2]
				{
					19347, -- [1]
					19348, -- [2]
					23100, -- [3]
				}, -- [3]
				{
					22277, -- [1]
					19361, -- [2]
					22299, -- [3]
				}, -- [4]
				{
					22268, -- [1]
					22276, -- [2]
					22499, -- [3]
				}, -- [5]
				{
					22300, -- [1]
					22278, -- [2]
					22271, -- [3]
				}, -- [6]
				{
					22272, -- [1]
					22301, -- [2]
					23105, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3568, -- [1]
				3567, -- [2]
				3566, -- [3]
			},
			["pvptalents"] = {
				3607, -- [1]
				3608, -- [2]
				3609, -- [3]
				3610, -- [4]
				665, -- [5]
				3615, -- [6]
				661, -- [7]
				662, -- [8]
				663, -- [9]
				664, -- [10]
				686, -- [11]
				3606, -- [12]
			},
		},
		[66] = {
			["talents"] = {
				{
					22428, -- [1]
					22558, -- [2]
					22430, -- [3]
				}, -- [1]
				{
					22431, -- [1]
					22604, -- [2]
					22594, -- [3]
				}, -- [2]
				{
					22179, -- [1]
					22180, -- [2]
					21811, -- [3]
				}, -- [3]
				{
					22433, -- [1]
					22434, -- [2]
					22435, -- [3]
				}, -- [4]
				{
					22705, -- [1]
					21795, -- [2]
					17601, -- [3]
				}, -- [5]
				{
					22189, -- [1]
					22438, -- [2]
					23087, -- [3]
				}, -- [6]
				{
					21201, -- [1]
					21202, -- [2]
					22645, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3471, -- [1]
				3469, -- [2]
				3470, -- [3]
			},
			["pvptalents"] = {
				93, -- [1]
				94, -- [2]
				97, -- [3]
				3475, -- [4]
				3474, -- [5]
				3472, -- [6]
				860, -- [7]
				861, -- [8]
				92, -- [9]
				91, -- [10]
				90, -- [11]
				844, -- [12]
			},
		},
		[257] = {
			["talents"] = {
				{
					22312, -- [1]
					19753, -- [2]
					19754, -- [3]
				}, -- [1]
				{
					22325, -- [1]
					22326, -- [2]
					19758, -- [3]
				}, -- [2]
				{
					22487, -- [1]
					22095, -- [2]
					22562, -- [3]
				}, -- [3]
				{
					21750, -- [1]
					21977, -- [2]
					19761, -- [3]
				}, -- [4]
				{
					19764, -- [1]
					22327, -- [2]
					21754, -- [3]
				}, -- [5]
				{
					19767, -- [1]
					19760, -- [2]
					19763, -- [3]
				}, -- [6]
				{
					21636, -- [1]
					21644, -- [2]
					23145, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3559, -- [1]
				3558, -- [2]
				3557, -- [3]
			},
			["pvptalents"] = {
				118, -- [1]
				121, -- [2]
				124, -- [3]
				127, -- [4]
				1927, -- [5]
				101, -- [6]
				108, -- [7]
				1242, -- [8]
				112, -- [9]
				115, -- [10]
			},
		},
		[258] = {
			["talents"] = {
				{
					22328, -- [1]
					22136, -- [2]
					22314, -- [3]
				}, -- [1]
				{
					22315, -- [1]
					23374, -- [2]
					21976, -- [3]
				}, -- [2]
				{
					23125, -- [1]
					23126, -- [2]
					23127, -- [3]
				}, -- [3]
				{
					23137, -- [1]
					23375, -- [2]
					21752, -- [3]
				}, -- [4]
				{
					22310, -- [1]
					22311, -- [2]
					21755, -- [3]
				}, -- [5]
				{
					21718, -- [1]
					21719, -- [2]
					21720, -- [3]
				}, -- [6]
				{
					21637, -- [1]
					21978, -- [2]
					21979, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3478, -- [1]
				3477, -- [2]
				3476, -- [3]
			},
			["pvptalents"] = {
				102, -- [1]
				106, -- [2]
				764, -- [3]
				763, -- [4]
				110, -- [5]
				119, -- [6]
				739, -- [7]
				128, -- [8]
				113, -- [9]
				3753, -- [10]
			},
		},
		[259] = {
			["pvpTalents"] = {
				145, -- [1]
				135, -- [2]
				139, -- [3]
				3619, -- [4]
				129, -- [5]
				853, -- [6]
				3483, -- [7]
				1208, -- [8]
				3421, -- [9]
				3451, -- [10]
				150, -- [11]
				3449, -- [12]
				142, -- [13]
				138, -- [14]
			},
			["talents"] = {
				{
					22337, -- [1]
					22338, -- [2]
					22339, -- [3]
				}, -- [1]
				{
					22331, -- [1]
					22332, -- [2]
					23022, -- [3]
				}, -- [2]
				{
					19239, -- [1]
					19240, -- [2]
					19241, -- [3]
				}, -- [3]
				{
					22340, -- [1]
					22122, -- [2]
					22123, -- [3]
				}, -- [4]
				{
					19245, -- [1]
					23037, -- [2]
					22115, -- [3]
				}, -- [5]
				{
					22343, -- [1]
					23015, -- [2]
					22344, -- [3]
				}, -- [6]
				{
					21186, -- [1]
					22133, -- [2]
					23174, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3453, -- [1]
				3456, -- [2]
				3461, -- [3]
			},
			["pvptalents"] = {
				3479, -- [1]
				3448, -- [2]
				131, -- [3]
				130, -- [4]
				137, -- [5]
				147, -- [6]
				830, -- [7]
				132, -- [8]
				144, -- [9]
				141, -- [10]
				3480, -- [11]
			},
		},
		[260] = {
			["talents"] = {
				{
					22118, -- [1]
					22119, -- [2]
					22120, -- [3]
				}, -- [1]
				{
					19236, -- [1]
					19237, -- [2]
					19238, -- [3]
				}, -- [2]
				{
					19239, -- [1]
					19240, -- [2]
					19241, -- [3]
				}, -- [3]
				{
					22121, -- [1]
					22122, -- [2]
					22123, -- [3]
				}, -- [4]
				{
					23077, -- [1]
					22114, -- [2]
					22115, -- [3]
				}, -- [5]
				{
					21990, -- [1]
					23128, -- [2]
					19250, -- [3]
				}, -- [6]
				{
					22125, -- [1]
					23075, -- [2]
					23175, -- [3]
				}, -- [7]
			},
			["pvpTalents"] = {
				145, -- [1]
				135, -- [2]
				139, -- [3]
				3619, -- [4]
				129, -- [5]
				853, -- [6]
				3483, -- [7]
				1208, -- [8]
				3421, -- [9]
				3451, -- [10]
				150, -- [11]
				3449, -- [12]
				142, -- [13]
				138, -- [14]
			},
			["pvptalenttrinkets"] = {
				3455, -- [1]
				3458, -- [2]
				3459, -- [3]
			},
			["pvptalents"] = {
				145, -- [1]
				135, -- [2]
				139, -- [3]
				3619, -- [4]
				129, -- [5]
				853, -- [6]
				3483, -- [7]
				1208, -- [8]
				3421, -- [9]
				3451, -- [10]
				150, -- [11]
				3449, -- [12]
				142, -- [13]
				138, -- [14]
			},
		},
		[261] = {
			["talents"] = {
				{
					19233, -- [1]
					19234, -- [2]
					19235, -- [3]
				}, -- [1]
				{
					22331, -- [1]
					22332, -- [2]
					22333, -- [3]
				}, -- [2]
				{
					19239, -- [1]
					19240, -- [2]
					19241, -- [3]
				}, -- [3]
				{
					22128, -- [1]
					22122, -- [2]
					22123, -- [3]
				}, -- [4]
				{
					23078, -- [1]
					23036, -- [2]
					22115, -- [3]
				}, -- [5]
				{
					22335, -- [1]
					19249, -- [2]
					22336, -- [3]
				}, -- [6]
				{
					22132, -- [1]
					23183, -- [2]
					21188, -- [3]
				}, -- [7]
			},
			["pvpTalents"] = {
				145, -- [1]
				135, -- [2]
				139, -- [3]
				3619, -- [4]
				129, -- [5]
				853, -- [6]
				3483, -- [7]
				1208, -- [8]
				3421, -- [9]
				3451, -- [10]
				150, -- [11]
				3449, -- [12]
				142, -- [13]
				138, -- [14]
			},
			["pvptalents"] = {
				3479, -- [1]
				3448, -- [2]
				131, -- [3]
				130, -- [4]
				137, -- [5]
				147, -- [6]
				830, -- [7]
				132, -- [8]
				144, -- [9]
				141, -- [10]
				3480, -- [11]
				3449, -- [12]
				142, -- [13]
				138, -- [14]
			},
		},
		[262] = {
			["talents"] = {
				{
					22356, -- [1]
					22357, -- [2]
					22358, -- [3]
				}, -- [1]
				{
					23108, -- [1]
					22139, -- [2]
					23190, -- [3]
				}, -- [2]
				{
					23162, -- [1]
					23163, -- [2]
					23164, -- [3]
				}, -- [3]
				{
					19271, -- [1]
					19272, -- [2]
					19273, -- [3]
				}, -- [4]
				{
					22144, -- [1]
					22172, -- [2]
					21966, -- [3]
				}, -- [5]
				{
					22145, -- [1]
					19266, -- [2]
					23111, -- [3]
				}, -- [6]
				{
					21198, -- [1]
					22153, -- [2]
					21675, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3596, -- [1]
				3597, -- [2]
				3598, -- [3]
			},
			["pvptalents"] = {
				3062, -- [1]
				731, -- [2]
				3491, -- [3]
				730, -- [4]
				728, -- [5]
				727, -- [6]
				3488, -- [7]
				3621, -- [8]
				3620, -- [9]
				3490, -- [10]
			},
		},
		[263] = {
			["talents"] = {
				{
					22354, -- [1]
					22355, -- [2]
					22353, -- [3]
				}, -- [1]
				{
					22636, -- [1]
					22150, -- [2]
					23109, -- [3]
				}, -- [2]
				{
					23165, -- [1]
					19260, -- [2]
					23166, -- [3]
				}, -- [3]
				{
					23089, -- [1]
					23090, -- [2]
					22171, -- [3]
				}, -- [4]
				{
					22144, -- [1]
					22149, -- [2]
					21966, -- [3]
				}, -- [5]
				{
					21973, -- [1]
					22352, -- [2]
					22351, -- [3]
				}, -- [6]
				{
					21970, -- [1]
					22977, -- [2]
					21972, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3553, -- [1]
				3552, -- [2]
				3551, -- [3]
			},
			["pvptalents"] = {
				3519, -- [1]
				3492, -- [2]
				725, -- [3]
				1944, -- [4]
				722, -- [5]
				721, -- [6]
				3487, -- [7]
				3489, -- [8]
				3623, -- [9]
				3622, -- [10]
			},
		},
		[264] = {
			["talents"] = {
				{
					19262, -- [1]
					19263, -- [2]
					19264, -- [3]
				}, -- [1]
				{
					19259, -- [1]
					22492, -- [2]
					21963, -- [3]
				}, -- [2]
				{
					19275, -- [1]
					23110, -- [2]
					22127, -- [3]
				}, -- [3]
				{
					22152, -- [1]
					22322, -- [2]
					22323, -- [3]
				}, -- [4]
				{
					22144, -- [1]
					19269, -- [2]
					21966, -- [3]
				}, -- [5]
				{
					19265, -- [1]
					21971, -- [2]
					21968, -- [3]
				}, -- [6]
				{
					21969, -- [1]
					21199, -- [2]
					22359, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3484, -- [1]
				3485, -- [2]
				3486, -- [3]
			},
			["pvptalents"] = {
				3756, -- [1]
				714, -- [2]
				718, -- [3]
				1930, -- [4]
				715, -- [5]
				3755, -- [6]
				707, -- [7]
				708, -- [8]
				712, -- [9]
				3520, -- [10]
				713, -- [11]
			},
		},
		[265] = {
			["talents"] = {
				{
					22039, -- [1]
					23140, -- [2]
					23141, -- [3]
				}, -- [1]
				{
					22044, -- [1]
					21180, -- [2]
					22089, -- [3]
				}, -- [2]
				{
					19280, -- [1]
					19285, -- [2]
					19286, -- [3]
				}, -- [3]
				{
					19279, -- [1]
					19292, -- [2]
					22046, -- [3]
				}, -- [4]
				{
					22047, -- [1]
					19291, -- [2]
					19288, -- [3]
				}, -- [5]
				{
					23139, -- [1]
					23159, -- [2]
					19295, -- [3]
				}, -- [6]
				{
					19284, -- [1]
					19281, -- [2]
					19293, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3498, -- [1]
				3497, -- [2]
				3496, -- [3]
			},
			["pvptalents"] = {
				9, -- [1]
				3740, -- [2]
				19, -- [3]
				20, -- [4]
				13, -- [5]
				10, -- [6]
				11, -- [7]
				12, -- [8]
				18, -- [9]
				17, -- [10]
				16, -- [11]
				15, -- [12]
			},
		},
		[266] = {
			["talents"] = {
				{
					19290, -- [1]
					22048, -- [2]
					23138, -- [3]
				}, -- [1]
				{
					22045, -- [1]
					21694, -- [2]
					23158, -- [3]
				}, -- [2]
				{
					19280, -- [1]
					19285, -- [2]
					19286, -- [3]
				}, -- [3]
				{
					22477, -- [1]
					22042, -- [2]
					23160, -- [3]
				}, -- [4]
				{
					22047, -- [1]
					19291, -- [2]
					19288, -- [3]
				}, -- [5]
				{
					23147, -- [1]
					23146, -- [2]
					21717, -- [3]
				}, -- [6]
				{
					23161, -- [1]
					22479, -- [2]
					23091, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3499, -- [1]
				3500, -- [2]
				3501, -- [3]
			},
			["pvptalents"] = {
				165, -- [1]
				3624, -- [2]
				3625, -- [3]
				3626, -- [4]
				3506, -- [5]
				3505, -- [6]
				3507, -- [7]
				154, -- [8]
				156, -- [9]
				158, -- [10]
				162, -- [11]
				1213, -- [12]
			},
		},
		[267] = {
			["talents"] = {
				{
					22038, -- [1]
					22090, -- [2]
					22040, -- [3]
				}, -- [1]
				{
					23148, -- [1]
					21695, -- [2]
					23157, -- [3]
				}, -- [2]
				{
					19280, -- [1]
					19285, -- [2]
					19286, -- [3]
				}, -- [3]
				{
					22480, -- [1]
					22043, -- [2]
					23143, -- [3]
				}, -- [4]
				{
					22047, -- [1]
					19291, -- [2]
					19288, -- [3]
				}, -- [5]
				{
					23155, -- [1]
					23156, -- [2]
					19295, -- [3]
				}, -- [6]
				{
					19284, -- [1]
					23144, -- [2]
					23092, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3493, -- [1]
				3494, -- [2]
				3495, -- [3]
			},
			["pvptalents"] = {
				155, -- [1]
				157, -- [2]
				161, -- [3]
				3741, -- [4]
				164, -- [5]
				3502, -- [6]
				3503, -- [7]
				3504, -- [8]
				3508, -- [9]
				3509, -- [10]
				3510, -- [11]
				159, -- [12]
			},
		},
		[268] = {
			["talents"] = {
				{
					23106, -- [1]
					19820, -- [2]
					20185, -- [3]
				}, -- [1]
				{
					19304, -- [1]
					19818, -- [2]
					19302, -- [3]
				}, -- [2]
				{
					22099, -- [1]
					22097, -- [2]
					19992, -- [3]
				}, -- [3]
				{
					19993, -- [1]
					19994, -- [2]
					19995, -- [3]
				}, -- [4]
				{
					20174, -- [1]
					23363, -- [2]
					20175, -- [3]
				}, -- [5]
				{
					19819, -- [1]
					20184, -- [2]
					22103, -- [3]
				}, -- [6]
				{
					22106, -- [1]
					22104, -- [2]
					22108, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3569, -- [1]
				3570, -- [2]
				3571, -- [3]
			},
			["pvptalents"] = {
				1958, -- [1]
				673, -- [2]
				672, -- [3]
				670, -- [4]
				669, -- [5]
				765, -- [6]
				668, -- [7]
				667, -- [8]
				666, -- [9]
				843, -- [10]
				671, -- [11]
			},
		},
		[269] = {
			["talents"] = {
				{
					23106, -- [1]
					19820, -- [2]
					20185, -- [3]
				}, -- [1]
				{
					19304, -- [1]
					19818, -- [2]
					19302, -- [3]
				}, -- [2]
				{
					22098, -- [1]
					19771, -- [2]
					22096, -- [3]
				}, -- [3]
				{
					19993, -- [1]
					23364, -- [2]
					19995, -- [3]
				}, -- [4]
				{
					23258, -- [1]
					20173, -- [2]
					20175, -- [3]
				}, -- [5]
				{
					22093, -- [1]
					23122, -- [2]
					22102, -- [3]
				}, -- [6]
				{
					22107, -- [1]
					22105, -- [2]
					21191, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3572, -- [1]
				3573, -- [2]
				3574, -- [3]
			},
			["pvptalents"] = {
				3052, -- [1]
				3734, -- [2]
				3737, -- [3]
				3745, -- [4]
				3744, -- [5]
				852, -- [6]
				3050, -- [7]
				77, -- [8]
				73, -- [9]
				675, -- [10]
			},
		},
		[270] = {
			["talents"] = {
				{
					19823, -- [1]
					19820, -- [2]
					20185, -- [3]
				}, -- [1]
				{
					19304, -- [1]
					19818, -- [2]
					19302, -- [3]
				}, -- [2]
				{
					22168, -- [1]
					22167, -- [2]
					22166, -- [3]
				}, -- [3]
				{
					19993, -- [1]
					22219, -- [2]
					19995, -- [3]
				}, -- [4]
				{
					23371, -- [1]
					20173, -- [2]
					20175, -- [3]
				}, -- [5]
				{
					23107, -- [1]
					22101, -- [2]
					22214, -- [3]
				}, -- [6]
				{
					22218, -- [1]
					22169, -- [2]
					22170, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3575, -- [1]
				3576, -- [2]
				3577, -- [3]
			},
			["pvptalents"] = {
				70, -- [1]
				676, -- [2]
				3732, -- [3]
				680, -- [4]
				683, -- [5]
				681, -- [6]
				678, -- [7]
				1928, -- [8]
				679, -- [9]
				682, -- [10]
			},
		},
		[70] = {
			["talents"] = {
				{
					22590, -- [1]
					22557, -- [2]
					22175, -- [3]
				}, -- [1]
				{
					22319, -- [1]
					22592, -- [2]
					22593, -- [3]
				}, -- [2]
				{
					22896, -- [1]
					22180, -- [2]
					21811, -- [3]
				}, -- [3]
				{
					22375, -- [1]
					22182, -- [2]
					22183, -- [3]
				}, -- [4]
				{
					22595, -- [1]
					22185, -- [2]
					22186, -- [3]
				}, -- [5]
				{
					23167, -- [1]
					22483, -- [2]
					23086, -- [3]
				}, -- [6]
				{
					22591, -- [1]
					22215, -- [2]
					22634, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3446, -- [1]
				3445, -- [2]
				3444, -- [3]
			},
			["pvptalents"] = {
				3055, -- [1]
				858, -- [2]
				641, -- [3]
				757, -- [4]
				756, -- [5]
				751, -- [6]
				752, -- [7]
				753, -- [8]
				754, -- [9]
				755, -- [10]
				81, -- [11]
			},
		},
		[102] = {
			["talents"] = {
				{
					22385, -- [1]
					22386, -- [2]
					22387, -- [3]
				}, -- [1]
				{
					19283, -- [1]
					18570, -- [2]
					18571, -- [3]
				}, -- [2]
				{
					22155, -- [1]
					22157, -- [2]
					22159, -- [3]
				}, -- [3]
				{
					21778, -- [1]
					18576, -- [2]
					18577, -- [3]
				}, -- [4]
				{
					18580, -- [1]
					21706, -- [2]
					21702, -- [3]
				}, -- [5]
				{
					22389, -- [1]
					21712, -- [2]
					22165, -- [3]
				}, -- [6]
				{
					21648, -- [1]
					21193, -- [2]
					21655, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3541, -- [1]
				3542, -- [2]
				3543, -- [3]
			},
			["pvptalents"] = {
				3731, -- [1]
				3728, -- [2]
				836, -- [3]
				822, -- [4]
				834, -- [5]
				3058, -- [6]
				1216, -- [7]
				857, -- [8]
				180, -- [9]
				182, -- [10]
				184, -- [11]
				185, -- [12]
			},
		},
		[71] = {
			["talents"] = {
				{
					22624, -- [1]
					22360, -- [2]
					22371, -- [3]
				}, -- [1]
				{
					19676, -- [1]
					22372, -- [2]
					22789, -- [3]
				}, -- [2]
				{
					22380, -- [1]
					22489, -- [2]
					19138, -- [3]
				}, -- [3]
				{
					15757, -- [1]
					22627, -- [2]
					22628, -- [3]
				}, -- [4]
				{
					22392, -- [1]
					22391, -- [2]
					22362, -- [3]
				}, -- [5]
				{
					22394, -- [1]
					22397, -- [2]
					22399, -- [3]
				}, -- [6]
				{
					21204, -- [1]
					22407, -- [2]
					21667, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3589, -- [1]
				3587, -- [2]
				3588, -- [3]
			},
			["pvptalents"] = {
				31, -- [1]
				34, -- [2]
				3521, -- [3]
				3522, -- [4]
				28, -- [5]
				3534, -- [6]
				29, -- [7]
				32, -- [8]
				33, -- [9]
			},
		},
		[103] = {
			["talents"] = {
				{
					22363, -- [1]
					22364, -- [2]
					22365, -- [3]
				}, -- [1]
				{
					19283, -- [1]
					18570, -- [2]
					18571, -- [3]
				}, -- [2]
				{
					22163, -- [1]
					22158, -- [2]
					22159, -- [3]
				}, -- [3]
				{
					21778, -- [1]
					18576, -- [2]
					18577, -- [3]
				}, -- [4]
				{
					21708, -- [1]
					18579, -- [2]
					21704, -- [3]
				}, -- [5]
				{
					21714, -- [1]
					21711, -- [2]
					22370, -- [3]
				}, -- [6]
				{
					21646, -- [1]
					21649, -- [2]
					21653, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3433, -- [1]
				3431, -- [2]
				3432, -- [3]
			},
			["pvptalents"] = {
				601, -- [1]
				602, -- [2]
				611, -- [3]
				612, -- [4]
				620, -- [5]
				3751, -- [6]
				3053, -- [7]
				820, -- [8]
				203, -- [9]
				202, -- [10]
				201, -- [11]
			},
		},
		[72] = {
			["talents"] = {
				{
					22632, -- [1]
					22633, -- [2]
					22491, -- [3]
				}, -- [1]
				{
					19676, -- [1]
					22625, -- [2]
					23093, -- [3]
				}, -- [2]
				{
					22379, -- [1]
					22381, -- [2]
					23372, -- [3]
				}, -- [3]
				{
					23097, -- [1]
					22627, -- [2]
					22382, -- [3]
				}, -- [4]
				{
					22383, -- [1]
					22393, -- [2]
					19140, -- [3]
				}, -- [5]
				{
					22396, -- [1]
					22398, -- [2]
					22400, -- [3]
				}, -- [6]
				{
					22405, -- [1]
					22402, -- [2]
					16037, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3591, -- [1]
				3592, -- [2]
				3590, -- [3]
			},
			["pvptalents"] = {
				25, -- [1]
				166, -- [2]
				170, -- [3]
				172, -- [4]
				177, -- [5]
				179, -- [6]
				3735, -- [7]
				3533, -- [8]
				3528, -- [9]
				1929, -- [10]
			},
		},
		[104] = {
			["talents"] = {
				{
					22419, -- [1]
					22418, -- [2]
					22420, -- [3]
				}, -- [1]
				{
					19283, -- [1]
					22916, -- [2]
					18571, -- [3]
				}, -- [2]
				{
					22163, -- [1]
					22156, -- [2]
					22159, -- [3]
				}, -- [3]
				{
					21778, -- [1]
					18576, -- [2]
					18577, -- [3]
				}, -- [4]
				{
					21709, -- [1]
					21707, -- [2]
					22388, -- [3]
				}, -- [5]
				{
					22423, -- [1]
					21713, -- [2]
					22390, -- [3]
				}, -- [6]
				{
					22426, -- [1]
					22427, -- [2]
					22425, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3465, -- [1]
				3463, -- [2]
				3464, -- [3]
			},
			["pvptalents"] = {
				193, -- [1]
				192, -- [2]
				50, -- [3]
				51, -- [4]
				52, -- [5]
				3750, -- [6]
				1237, -- [7]
				49, -- [8]
				842, -- [9]
				194, -- [10]
				195, -- [11]
				197, -- [12]
				196, -- [13]
			},
		},
		[73] = {
			["talents"] = {
				{
					15760, -- [1]
					15759, -- [2]
					15774, -- [3]
				}, -- [1]
				{
					22373, -- [1]
					22629, -- [2]
					22409, -- [3]
				}, -- [2]
				{
					22378, -- [1]
					22626, -- [2]
					23260, -- [3]
				}, -- [3]
				{
					23096, -- [1]
					23261, -- [2]
					22488, -- [3]
				}, -- [4]
				{
					22384, -- [1]
					22631, -- [2]
					22800, -- [3]
				}, -- [5]
				{
					22395, -- [1]
					22544, -- [2]
					22401, -- [3]
				}, -- [6]
				{
					21204, -- [1]
					22406, -- [2]
					23099, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3595, -- [1]
				3594, -- [2]
				3593, -- [3]
			},
			["pvptalents"] = {
				833, -- [1]
				167, -- [2]
				168, -- [3]
				171, -- [4]
				173, -- [5]
				845, -- [6]
				178, -- [7]
				831, -- [8]
				175, -- [9]
				24, -- [10]
			},
		},
		[581] = {
			["talents"] = {
				{
					22502, -- [1]
					22503, -- [2]
					22504, -- [3]
				}, -- [1]
				{
					22505, -- [1]
					22766, -- [2]
					22507, -- [3]
				}, -- [2]
				{
					22324, -- [1]
					22541, -- [2]
					22540, -- [3]
				}, -- [3]
				{
					22508, -- [1]
					22509, -- [2]
					22770, -- [3]
				}, -- [4]
				{
					22546, -- [1]
					22510, -- [2]
					22511, -- [3]
				}, -- [5]
				{
					22512, -- [1]
					22513, -- [2]
					22768, -- [3]
				}, -- [6]
				{
					22543, -- [1]
					22548, -- [2]
					21902, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3546, -- [1]
				3545, -- [2]
				3544, -- [3]
			},
			["pvptalents"] = {
				3429, -- [1]
				3423, -- [2]
				814, -- [3]
				815, -- [4]
				816, -- [5]
				3430, -- [6]
				1948, -- [7]
				1220, -- [8]
				802, -- [9]
				3727, -- [10]
				819, -- [11]
			},
		},
		[105] = {
			["talents"] = {
				{
					18569, -- [1]
					18574, -- [2]
					18572, -- [3]
				}, -- [1]
				{
					19283, -- [1]
					18570, -- [2]
					18571, -- [3]
				}, -- [2]
				{
					22366, -- [1]
					22367, -- [2]
					22160, -- [3]
				}, -- [3]
				{
					21778, -- [1]
					18576, -- [2]
					18577, -- [3]
				}, -- [4]
				{
					21710, -- [1]
					21705, -- [2]
					22421, -- [3]
				}, -- [5]
				{
					21716, -- [1]
					18585, -- [2]
					22422, -- [3]
				}, -- [6]
				{
					22403, -- [1]
					21651, -- [2]
					22404, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3, -- [1]
				4, -- [2]
				5, -- [3]
			},
			["pvptalents"] = {
				690, -- [1]
				691, -- [2]
				692, -- [3]
				697, -- [4]
				700, -- [5]
				835, -- [6]
				838, -- [7]
				839, -- [8]
				1215, -- [9]
				1217, -- [10]
				3048, -- [11]
				3752, -- [12]
				59, -- [13]
			},
		},
		[577] = {
			["talents"] = {
				{
					21854, -- [1]
					22493, -- [2]
					22416, -- [3]
				}, -- [1]
				{
					21857, -- [1]
					22765, -- [2]
					22799, -- [3]
				}, -- [2]
				{
					22909, -- [1]
					22494, -- [2]
					21862, -- [3]
				}, -- [3]
				{
					21863, -- [1]
					21864, -- [2]
					21865, -- [3]
				}, -- [4]
				{
					21866, -- [1]
					21867, -- [2]
					21868, -- [3]
				}, -- [5]
				{
					21869, -- [1]
					21870, -- [2]
					22767, -- [3]
				}, -- [6]
				{
					21900, -- [1]
					21901, -- [2]
					22547, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3426, -- [1]
				3427, -- [2]
				3428, -- [3]
			},
			["pvptalents"] = {
				812, -- [1]
				806, -- [2]
				807, -- [3]
				809, -- [4]
				813, -- [5]
				1218, -- [6]
				810, -- [7]
				1206, -- [8]
				1204, -- [9]
				805, -- [10]
				811, -- [11]
			},
		},
		[65] = {
			["talents"] = {
				{
					17565, -- [1]
					17567, -- [2]
					17569, -- [3]
				}, -- [1]
				{
					22176, -- [1]
					17575, -- [2]
					17577, -- [3]
				}, -- [2]
				{
					22179, -- [1]
					22180, -- [2]
					21811, -- [3]
				}, -- [3]
				{
					22181, -- [1]
					17591, -- [2]
					17593, -- [3]
				}, -- [4]
				{
					17597, -- [1]
					17599, -- [2]
					22164, -- [3]
				}, -- [5]
				{
					23191, -- [1]
					22190, -- [2]
					22484, -- [3]
				}, -- [6]
				{
					21668, -- [1]
					21671, -- [2]
					21203, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3549, -- [1]
				3548, -- [2]
				3547, -- [3]
			},
			["pvptalents"] = {
				87, -- [1]
				859, -- [2]
				88, -- [3]
				3618, -- [4]
				640, -- [5]
				689, -- [6]
				642, -- [7]
				82, -- [8]
				85, -- [9]
				86, -- [10]
			},
		},
		[256] = {
			["talents"] = {
				{
					19752, -- [1]
					22313, -- [2]
					22329, -- [3]
				}, -- [1]
				{
					22315, -- [1]
					22316, -- [2]
					19758, -- [3]
				}, -- [2]
				{
					22440, -- [1]
					22094, -- [2]
					19755, -- [3]
				}, -- [3]
				{
					19759, -- [1]
					19769, -- [2]
					19761, -- [3]
				}, -- [4]
				{
					22330, -- [1]
					19765, -- [2]
					19766, -- [3]
				}, -- [5]
				{
					22161, -- [1]
					19760, -- [2]
					19763, -- [3]
				}, -- [6]
				{
					21183, -- [1]
					21184, -- [2]
					22976, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3556, -- [1]
				3554, -- [2]
				3555, -- [3]
			},
			["pvptalents"] = {
				109, -- [1]
				126, -- [2]
				114, -- [3]
				111, -- [4]
				1244, -- [5]
				98, -- [6]
				100, -- [7]
				117, -- [8]
				855, -- [9]
				123, -- [10]
			},
		},
		[64] = {
			["talents"] = {
				{
					22457, -- [1]
					22460, -- [2]
					22463, -- [3]
				}, -- [1]
				{
					22442, -- [1]
					22443, -- [2]
					23073, -- [3]
				}, -- [2]
				{
					22444, -- [1]
					22445, -- [2]
					22447, -- [3]
				}, -- [3]
				{
					22452, -- [1]
					22466, -- [2]
					22469, -- [3]
				}, -- [4]
				{
					22446, -- [1]
					22448, -- [2]
					22471, -- [3]
				}, -- [5]
				{
					22454, -- [1]
					23176, -- [2]
					22473, -- [3]
				}, -- [6]
				{
					21632, -- [1]
					22309, -- [2]
					21634, -- [3]
				}, -- [7]
			},
			["pvptalenttrinkets"] = {
				3584, -- [1]
				3586, -- [2]
				3585, -- [3]
			},
			["pvptalents"] = {
				632, -- [1]
				3532, -- [2]
				3443, -- [3]
				633, -- [4]
				57, -- [5]
				58, -- [6]
				634, -- [7]
				3516, -- [8]
				66, -- [9]
				67, -- [10]
				68, -- [11]
			},
		},
	};
	function GetTalentInfoForSpecID(specID, tier, column)
		for specIndex=1,GetNumSpecializations() do
			local playerSpecID = GetSpecializationInfo(specIndex);
			if playerSpecID == specID then
				return GetTalentInfoBySpecialization(specIndex, tier, column);
			end
		end

		if BtWLoadoutsSpecInfo[specID] then
			return GetTalentInfoByID(BtWLoadoutsSpecInfo[specID].talents[tier][column]);
		end

		if specInfo[specID] then
			return GetTalentInfoByID(specInfo[specID].talents[tier][column]);
		end
	end
	function GetPvPTrinketTalentInfo(specID, index)
		local playerSpecID = GetSpecializationInfo(GetSpecialization());
		if playerSpecID == specID then
			local slotInfo = GetPvpTalentSlotInfo(1);
			if slotInfo and slotInfo.availableTalentIDs[index] then
				return GetPvpTalentInfoByID(slotInfo.availableTalentIDs[index]);
			end
		end

		if BtWLoadoutsSpecInfo[specID] and BtWLoadoutsSpecInfo[specID].pvptalenttrinkets and BtWLoadoutsSpecInfo[specID].pvptalenttrinkets[index] then
			return GetPvpTalentInfoByID(BtWLoadoutsSpecInfo[specID].pvptalenttrinkets[index]);
		end

		if specInfo[specID] and specInfo[specID].pvptalenttrinkets and specInfo[specID].pvptalenttrinkets[index] then
			return GetPvpTalentInfoByID(specInfo[specID].pvptalenttrinkets[index]);
		end
	end
	function GetPvPTalentInfoForSpecID(specID, index)
		local playerSpecID = GetSpecializationInfo(GetSpecialization());
		if playerSpecID == specID then
			local slotInfo = GetPvpTalentSlotInfo(2);
			if slotInfo and slotInfo.availableTalentIDs[index] then
				return GetPvpTalentInfoByID(slotInfo.availableTalentIDs[index]);
			end
		end

		if BtWLoadoutsSpecInfo[specID] and BtWLoadoutsSpecInfo[specID].pvptalents and BtWLoadoutsSpecInfo[specID].pvptalents[index] then
			return GetPvpTalentInfoByID(BtWLoadoutsSpecInfo[specID].pvptalents[index]);
		end

		if specInfo[specID] and specInfo[specID].pvptalents and specInfo[specID].pvptalents[index] then
			return GetPvpTalentInfoByID(specInfo[specID].pvptalents[index]);
		end
	end
	Internal.GetTalentInfoForSpecID = GetTalentInfoForSpecID
	Internal.GetPvPTrinketTalentInfo = GetPvPTrinketTalentInfo
	Internal.GetPvPTalentInfoForSpecID = GetPvPTalentInfoForSpecID
end
local GetEssenceInfoByID, GetEssenceInfoForRole;
do
	local essenceInfo = {
		nil, -- [1]
		{
			["ID"] = 2,
			["name"] = "Azeroth's Undying Gift",
			["icon"] = 2967107,
		}, -- [2]
		{
			["ID"] = 3,
			["name"] = "Sphere of Suppression",
			["icon"] = 2065602,
		}, -- [3]
		{
			["ID"] = 4,
			["name"] = "Worldvein Resonance",
			["icon"] = 1830317,
		}, -- [4]
		{
			["ID"] = 5,
			["name"] = "Essence of the Focusing Iris",
			["icon"] = 2967111,
		}, -- [5]
		{
			["ID"] = 6,
			["name"] = "Purification Protocol",
			["icon"] = 2967103,
		}, -- [6]
		{
			["ID"] = 7,
			["name"] = "Anima of Life and Death",
			["icon"] = 2967105,
		}, -- [7]
		nil, -- [8]
		nil, -- [9]
		nil, -- [10]
		nil, -- [11]
		{
			["ID"] = 12,
			["name"] = "The Crucible of Flame",
			["icon"] = 3015740,
		}, -- [12]
		{
			["ID"] = 13,
			["name"] = "Nullification Dynamo",
			["icon"] = 3015741,
		}, -- [13]
		{
			["ID"] = 14,
			["name"] = "Condensed Life-Force",
			["icon"] = 2967113,
		}, -- [14]
		{
			["ID"] = 15,
			["name"] = "Ripple in Space",
			["icon"] = 2967109,
		}, -- [15]
		{
			["ID"] = 16,
			["name"] = "Unwavering Ward",
			["icon"] = 3193842,
		}, -- [16]
		{
			["ID"] = 17,
			["name"] = "The Ever-Rising Tide",
			["icon"] = 2967108,
		}, -- [17]
		{
			["ID"] = 18,
			["name"] = "Artifice of Time",
			["icon"] = 2967112,
		}, -- [18]
		{
			["ID"] = 19,
			["name"] = "The Well of Existence",
			["icon"] = 516796,
		}, -- [19]
		{
			["ID"] = 20,
			["name"] = "Life-Binder's Invocation",
			["icon"] = 2967106,
		}, -- [20]
		{
			["ID"] = 21,
			["name"] = "Vitality Conduit",
			["icon"] = 2967100,
		}, -- [21]
		{
			["ID"] = 22,
			["name"] = "Vision of Perfection",
			["icon"] = 3015743,
		}, -- [22]
		{
			["ID"] = 23,
			["name"] = "Blood of the Enemy",
			["icon"] = 2032580,
		}, -- [23]
		{
			["ID"] = 24,
			["name"] = "Spirit of Preservation",
			["icon"] = 2967101,
		}, -- [24]
		{
			["ID"] = 25,
			["name"] = "Aegis of the Deep",
			["icon"] = 2967110,
		}, -- [25]
		nil, -- [26]
		{
			["ID"] = 27,
			["name"] = "Memory of Lucid Dreams",
			["icon"] = 2967104,
		}, -- [27]
		{
			["ID"] = 28,
			["name"] = "The Unbound Force",
			["icon"] = 2967102,
		}, -- [28]
		nil, -- [29]
		nil, -- [30]
		nil, -- [31]
		{
			["ID"] = 32,
			["name"] = "Conflict and Strife",
			["icon"] = 3015742,
		}, -- [32]
		{
			["ID"] = 33,
			["name"] = "Touch of the Everlasting",
			["icon"] = 3193847,
		}, -- [33]
		{
			["ID"] = 34,
			["name"] = "Strength of the Warden",
			["icon"] = 3193846,
		}, -- [34]
		{
			["ID"] = 35,
			["name"] = "Breath of the Dying",
			["icon"] = 3193844,
		}, -- [35]
		{
			["ID"] = 36,
			["name"] = "Spark of Inspiration",
			["icon"] = 3193843,
		}, -- [36]
		{
			["ID"] = 37,
			["name"] = "The Formless Void",
			["icon"] = 3193845,
		}, -- [37]
	}
	local roleInfo = {
		["DAMAGER"] = {
			["essences"] = {
				23, -- [1]
				35, -- [2]
				14, -- [3]
				32, -- [4]
				5, -- [5]
				27, -- [6]
				6, -- [7]
				15, -- [8]
				36, -- [9]
				12, -- [10]
				37, -- [11]
				28, -- [12]
				22, -- [13]
				4, -- [14]
			},
		},
		["TANK"] = {
			["essences"] = {
				25, -- [1]
				7, -- [2]
				2, -- [3]
				32, -- [4]
				27, -- [5]
				13, -- [6]
				15, -- [7]
				3, -- [8]
				34, -- [9]
				12, -- [10]
				37, -- [11]
				33, -- [12]
				22, -- [13]
				4, -- [14]
			},
		},
		["HEALER"] = {
			["essences"] = {
				18, -- [1]
				32, -- [2]
				20, -- [3]
				27, -- [4]
				15, -- [5]
				24, -- [6]
				12, -- [7]
				17, -- [8]
				37, -- [9]
				19, -- [10]
				16, -- [11]
				22, -- [12]
				21, -- [13]
				4, -- [14]
			},
		},
	};
	function GetEssenceInfoByID(essenceID)
		local essence = GetEssenceInfo(essenceID);
		if not essence then
			essence = BtWLoadoutsEssenceInfo and BtWLoadoutsEssenceInfo[essenceID] or essenceInfo[essenceID];
		end
		return essence;
	end
	function GetEssenceInfoForRole(role, index)
		if BtWLoadoutsRoleInfo[role] and BtWLoadoutsRoleInfo[role].essences and BtWLoadoutsRoleInfo[role].essences[index] then
			return GetEssenceInfoByID(BtWLoadoutsRoleInfo[role].essences[index]);
		end

		if roleInfo[role] and roleInfo[role].essences and roleInfo[role].essences[index] then
			return GetEssenceInfoByID(roleInfo[role].essences[index]);
		end
	end
	Internal.GetEssenceInfoByID = GetEssenceInfoByID
	Internal.GetEssenceInfoForRole = GetEssenceInfoForRole
end
function Internal.GetCharacterInfo(character)
	return BtWLoadoutsCharacterInfo and BtWLoadoutsCharacterInfo[character];
end
function Internal.UpdatePlayerInfo()
    local name, realm = UnitFullName("player");
    local class = select(2, UnitClass("player"));
    local race = select(3, UnitRace("player"));
    local sex = UnitSex("player") - 2;

    BtWLoadoutsCharacterInfo[realm .. "-" .. name] = {name = name, realm = GetRealmName(), class = class, race = race, sex = sex};
end
-- Checks if the player can switch to specID, used to check if loadouts are valid
function Internal.CanSwitchToSpecialization(specID)
	local playerClass = select(2, UnitClass("player"));
	local specClass = select(6, GetSpecializationInfoByID(specID));
	if playerClass ~= specClass then
		return false
	end
	local specIndex = GetSpecialization()
	if specIndex == nil then
		return false
	end

	if select(2, GetInstanceInfo()) == "arena" then
		-- Can not switch specs in arena
		return GetSpecializationInfo(specIndex) == specID
	elseif select(2, GetInstanceInfo()) == "battleground" then
		-- You can only switch specs in bgs unless you go from or to healer
		local currentRole = GetSpecializationRole(specIndex)
		local targetRole = GetSpecializationRoleByID(specID)

		return (currentRole == targetRole) or not (currentRole == "HEALER" or targetRole == "HEALER")
	end

	return true
end